
from typing import Optional

from datetime import datetime, UTC


class Token:
    """
    Representation of an authentication token (such as a barcode) associated with a user.

    :param _id: The token ID in the database.
    :param user_id: The ID of the user this token belongs to.
    :param token: The token secret.
    :param name: The display name of the token:
    :param date: The date the tokenw as created.
    """

    def __init__(self,
                 _id: int,
                 user_id: int,
                 token: str,
                 name: Optional[str] = None,
                 date: Optional[datetime] = None) -> None:
        self.id: int = _id
        self.user_id: int = user_id
        self.token: str = token
        self.name = name
        self.date = date

    def __eq__(self, other) -> bool:
        if not isinstance(other, Token):
            return False
        return self.id == other.id and \
            self.user_id == other.user_id and \
            self.token == other.token and \
            self.name == other.name and \
            self.date == other.date

    def __hash__(self) -> int:
        return hash((self.id, self.user_id, self.token, self.name, self.date))
