from bottle import get, post, redirect, request

from matemat.db import MatematDatabase
from matemat.webserver import session
from matemat.webserver import config as c
from matemat.webserver.template import Notification
from matemat.util.currency_format import format_chf


@get('/buy')
@post('/buy')
def buy():
    """
    The purchasing mechanism.  Called by the user clicking an item on the product list.
    """
    config = c.get_app_config()
    session_id: str = session.start()
    # If no user is logged in, redirect to the main page, as a purchase must always be bound to a user
    if not session.has(session_id, 'authenticated_user'):
        redirect('/')
    authlevel: int = session.get(session_id, 'authentication_level')
    # Connect to the database
    with MatematDatabase(config['DatabaseFile']) as db:
        # Fetch the authenticated user from the database
        uid: int = session.get(session_id, 'authenticated_user')
        user = db.get_user(uid)
        # Read the product from the database, identified by the product ID passed as request argument
        if 'pid' in request.params:
            pid = int(str(request.params.pid))
            product = db.get_product(pid)
            if c.get_dispenser().dispense(product, 1):
                price = product.price_member if user.is_member else product.price_non_member
                if 'price' in request.params:
                    price = int(str(request.params.price))
                # Create a consumption entry for the (user, product) combination
                db.increment_consumption(user, product, price)
                stock_provider = c.get_stock_provider()
                if stock_provider.needs_update():
                    stock_provider.update_stock(product, -1)
                # Show notification on next page load
                Notification.success(
                    f'Purchased <strong>{product.name}</strong> for <strong>{format_chf(price)}</strong>', decay=True)
                # Logout user if configured, logged in via touchkey and no price entry input was shown
                if user.logout_after_purchase and authlevel < 3 and not product.custom_price:
                    redirect('/logout')
    # Redirect to the main page (where this request should have come from)
    redirect('/')
